"""Converters for zope.schema-based datatypes.
"""
import grok
from zope.component import getMultiAdapter
from zope.publisher.browser import TestRequest
try:
    from zope.app.form.browser.interfaces import ITerms
except ImportError:
    from zope.browser.interfaces import ITerms
from zope.schema.interfaces import IBool, IText, IInt, IChoice
from waeup.interfaces import ISchemaTypeConverter

class Converter(grok.Adapter):
    """Base for string-or-none to zope.schema field converters.
    """
    grok.baseclass()
    grok.provides(ISchemaTypeConverter)
    
    def __init__(self, context):
        """Create a converter with context, a zope.schema.Field, as context.
        """
        self.context = context

    def _convertValueFromString(self, string):
        """You must at least override this method to build a working
           converter.
        """
        raise NotImplementedError('method not implemented')
    
    def fromString(self, string=None, strict=True):
        """Convert ``string`` to value according to assigned field type.
        """
        result = None
        if string is None:
            if self.context.required is True:
                result = self.context.default
            else:
                result = self.context.missing_value
        else:
            result = self._convertValueFromString(string)
        if strict:
            self.context.validate(result)
        return result

    def _convertValueToString(self, value):
        return str(value)

    def toString(self, value, strict=True):
        if strict:
            self.context.validate(value)
        if value == self.context.missing_value:
            return None
        return self._convertValueToString(value)

class BoolConverter(Converter):
    grok.context(IBool)
    grok.provides(ISchemaTypeConverter)

    def _convertValueFromString(self, string):
        if string is None:
            return None
        if string.lower() in ['1', 'true', 'yes']:
            return True
        return False

    def _convertValueToString(self, value):
        if value is None:
            return None
        if value:
            return '1'
        return '0'
    
class TextConverter(Converter):
    grok.context(IText)
    grok.provides(ISchemaTypeConverter)

    def _convertValueFromString(self, string):
        return unicode(string)

class IntConverter(Converter):
    grok.context(IInt)
    grok.provides(ISchemaTypeConverter)

    def _convertValueFromString(self, string):
        return int(string)

class ChoiceConverter(Converter):
    grok.context(IChoice)
    grok.provides(ISchemaTypeConverter)

    def __init__(self, context):
        self.context = context
        try:
            self.terms = getMultiAdapter(
                (self.context.source, TestRequest()), ITerms)
        except:
            self.terms = None

    def _convertValueFromString(self, string):
        if self.terms is not None:
            return self.terms.getValue(string)
        return self.context.source.getTermByToken(string).value

    def _convertValueToString(self, value):
        if self.terms is not None:
            return self.terms.getTerm(value).token
        return str(value)
